
import pandas as pd
import matplotlib.pyplot as plt
import numpy as np
import scipy
import math
import statistics
import os
from sklearn.linear_model import LinearRegression
from scipy import stats

import statsmodels.api as sm
import statsmodels.formula.api as smf
from statsmodels.graphics.tsaplots import plot_acf
from statsmodels.graphics.tsaplots import plot_pacf

plt.rcParams['text.color'] = 'black'  # Set the text color to black
plt.rcParams['font.family'] = 'Times new roman'


os.getcwd()

#Data
data = pd.read_excel("main_data.xlsx")
data['date'] = pd.to_datetime(data['date'])

#Data manipulering
#data.iloc[871,4]=0.0039



data["incomeReturn"] = data["Carry"] + data["Rf"]
data["priceReturn"] = data["futucom"] - data["incomeReturn"] 

Indexes = data.copy()
Indexes.iloc[:,1:] = np.cumprod(Indexes.iloc[:, 1:]+1) 
data["tenyearreturn_futu"] = np.zeros(len(data))
data["tenyearreturn_spot"] = np.zeros(len(data))
data["tenyearincomereturn"] = np.zeros(len(data))
data["tenyearpriceReturn"] = np.zeros(len(data))


for i in range(len(data)-120+1):
    if i==0:
        data["tenyearreturn_futu"][i+120-1] = Indexes.futucom[i+120-1]-1
        data["tenyearreturn_spot"][i+120-1] = Indexes.spotcom[i+120-1]-1
        data["tenyearincomereturn"][i+120-1] = Indexes.incomeReturn[i+120-1]-1
        data["tenyearpriceReturn"][i+120-1] = Indexes.priceReturn[i+120-1]-1
    
        
    else:
        data["tenyearreturn_futu"][i+120-1] = Indexes.futucom[i+120-1]/Indexes.futucom[i-1]-1
        data["tenyearreturn_spot"][i+120-1] = Indexes.spotcom[i+120-1]/Indexes.spotcom[i-1]-1
        data["tenyearincomereturn"][i+120-1] = Indexes.incomeReturn[i+120-1]/Indexes.incomeReturn[i-1]-1
        data["tenyearpriceReturn"][i+120-1] = Indexes.priceReturn[i+120-1]/Indexes.priceReturn[i-1]-1




#Annulizing from ten year 

data["tenyearreturn_futu"] = (1+data["tenyearreturn_futu"])**(1/10)-1
data["tenyearreturn_spot"] = (1+data["tenyearreturn_spot"])**(1/10)-1
data["tenyearincomereturn"] = (1+data["tenyearincomereturn"])**(1/10)-1
data["tenyearpriceReturn"] = (1+data["tenyearpriceReturn"])**(1/10)-1



#
monthly_data = data.iloc[:, [0,5,6,13,14,15,20, 21]]
corr_monthly = monthly_data.corr()


###############
#10 year

tenyear_data = data.iloc[120-1:, [0,22, 23, 24, 25]]

corr_tenyear = tenyear_data.corr()


#Reg og plot av all data monthly figure 2 from here FIG with 4 in one
#Reg og plot av all data monthly figure 4

#rcParams["figure.figsize"] = (10,6)
fig, axs = plt.subplots(2,2)
fig.suptitle("Regressions on Commodity Futures Total Returns", fontsize = 26)

y = monthly_data.futucom.to_numpy()

x = monthly_data.incomeReturn.to_numpy()

reg_monthly = LinearRegression().fit(x.reshape(len(x), 1), y)

axs[0,0].scatter(x, y, color='#B22222', s = 3 )
axs[0,0].plot(x, reg_monthly.predict(x.reshape(len(x), 1)), color='#2E8B57', linewidth = 2.5,  label='Total = 0.0065 + 0.14 Income, R^2 = 0.002')

#axs[0,0].rcParams["figure.figsize"] = (10,6)
axs[0,0].grid()
#axs[0,0].xticks(fontsize = 16)
#axs[0,0].yticks(fontsize = 16)

axs[0,0].set_ylabel('Income Return', fontsize = 16)
#axs[0,0].pltxticks(fontsize = 12)
#axs[0,0].pltyticks(fontsize = 12)
#axs[0,0].set_xlabel('1-Month', fontsize = 12)
#axs[0,0].ylabel('Total return')
#axs[0,0].set_title('Linear Regression Fit')
axs[0,0].legend(loc = "lower right")

print("R square is : " + str(reg_monthly.score(x.reshape(len(x), 1), y)))




#add constant to predictor variables
x = sm.add_constant(x)

#fit linear regression model
model = sm.OLS(y, x).fit()
#view model summary
print(model.summary())



#Reg og plot av all data monthly figure 3




y = monthly_data.futucom.to_numpy()

x = monthly_data.spotcom.to_numpy()

reg_monthly_price = LinearRegression().fit(x.reshape(len(x), 1), y)

axs[1,0].scatter(x, y, color='#0072B2', s = 3)
axs[1,0].plot(x, reg_monthly_price.predict(x.reshape(len(x), 1)), color='#2E8B57', linewidth = 2.5,  label='Total = 0.0023 + 0.9324 Price, R^2 = 0.919')

#axs[0,1].rcParams["figure.figsize"] = (10,6)
axs[1,0].grid()
#axs[0,0].xticks(fontsize = 16)
#axs[0,0].yticks(fontsize = 16)
axs[1,0].set_xlabel('1-Month', fontsize = 16)
axs[1,0].set_ylabel('Price Return', fontsize = 16)
#axs[0,1].title('Linear Regression Fit')
axs[1,0].legend(loc = "lower right")

print("R square is : " + str(reg_monthly_price.score(x.reshape(len(x), 1), y)))




#add constant to predictor variables
x = sm.add_constant(x)

#fit linear regression model
model = sm.OLS(y, x).fit()

#view model summary
print(model.summary())


############### ten years

y = tenyear_data.tenyearreturn_futu.to_numpy()

x = tenyear_data.tenyearincomereturn.to_numpy()

reg_tenyear = LinearRegression().fit(x.reshape(len(x), 1), y)

axs[0,1].scatter(x, y, color='#B22222', s=3)
axs[0,1].plot(x, reg_tenyear.predict(x.reshape(len(x), 1)), color='#2E8B57', linewidth = 2.5,  label='Total = 0.0319 + 1.14 Income, R^2 = 0.561')

#axs[0,1].rcParams["figure.figsize"] = (10,6)
axs[0,1].grid()
#axs[0,0].xticks(fontsize = 16)
#[0,0].yticks(fontsize = 16)
#axs[0,1].set_xlabel('Y2')
#axs[0,1].set_ylabel("X1")
#axs[0,1].title('Linear Regression Fit')
axs[0,1].legend(loc = "lower right")

print("R square is : " + str(reg_tenyear.score(x.reshape(len(x), 1), y)))




#add constant to predictor variables
x = sm.add_constant(x)

#fit linear regression model
model = sm.OLS(y, x).fit()


model = model.get_robustcov_results(cov_type='HAC', maxlags=1, use_t=True)

#view model summary
print(model.summary())



#Reg og plot av all data 10 year figure 3




y = tenyear_data.tenyearreturn_futu.to_numpy()

x = tenyear_data.tenyearreturn_spot.to_numpy()

reg_tenyear_price = LinearRegression().fit(x.reshape(len(x), 1), y)

axs[1,1].scatter(x, y, color='#0072B2', s = 3)
axs[1,1].plot(x, reg_tenyear_price.predict(x.reshape(len(x), 1)), color='#2E8B57', linewidth = 2.5, label='Total = 0.0238 + 1.17 Price, R^2 = 0.560')

#axs[1,1].rcParams["figure.figsize"] = (10,6)
axs[1,1].grid()
axs[1,1].set_xlabel('10-year', fontsize = 16)
#axs[0,0].xticks(fontsize = 16)
#axs[0,0].yticks(fontsize = 16)
#axs[1,1].ylabel('Total return')
#axs[1,1].title('Linear Regression Fit')
axs[1,1].legend(loc = "lower right")

print("R square is : " + str(reg_tenyear_price.score(x.reshape(len(x), 1), y)))




#add constant to predictor variables
x = sm.add_constant(x)

#fit linear regression model
model = sm.OLS(y, x).fit()


model = model.get_robustcov_results(cov_type='HAC', maxlags=1, use_t=True)


#view model summary
print(model.summary())


fig.savefig("RegTot4", bbox_inches= "tight")

